@@ -18,6 +18,7 @@ from bakplane.bakplane_pb2 import (
 date_constraint_t = typing.Union[
     typing.Tuple[datetime.datetime, datetime.datetime],
     typing.Tuple[datetime.datetime, datetime.datetime, str],
+    typing.Tuple[datetime.datetime, datetime.datetime, str, str],
     datetime.datetime,
 ]
 
@@ -33,11 +34,10 @@ def flatten_date_constraints(c: date_constraint_t) -> typing.List[datetime.datet
     elif isinstance(c, tuple):
         if len(c) == 2:
             return [c[0], c[1]]
-        elif len(c) == 3:
+        elif len(c) == 3 or len(c) == 4:
             results = set()
 
-            start, end, freq = c
-            ranges = pd.bdate_range(start=start, end=end, freq=freq)
+            ranges = pd.bdate_range(start=c[0], end=c[1], freq=c[2])
             for r in ranges:
                 results.add(r.to_pydatetime())
             return list(results)
@@ -87,6 +87,9 @@ class ReadRequestBuildingBlocks:
     selectable: FromClause
     constraints: typing.List[typing.Any]
 
+    effective_dating_range_semantics: str
+    valid_dating_range_semantics: str
+
     effective_dating_table: Table = None
     valid_dating_table: Table = None
 
