@@ -125,6 +125,19 @@ def __build_read_all_query(
     return f'select {", ".join(r.columns)} from {res.context["schema_name"]}.{res.context["table_name"]}'
 
 
+def __get_range_semantics_ops(semantics: str) -> typing.Tuple[str, str]:
+    if semantics == '[)':
+        return '>=', '<'
+    elif semantics == '[]':
+        return '>=', '<='
+    elif semantics == '(]':
+        return '>', '<='
+    elif semantics == '()':
+        return '>', '<'
+    else:
+        raise RuntimeError(f'`{semantics}` is not a valid range semantics.')
+
+
 def __add_effective_dating_constraint(blocks: ReadRequestBuildingBlocks, res: ResolveResourceIntentResponse,
                                       r: ReadRequest):
     a = blocks.asset_table
@@ -132,60 +145,61 @@ def __add_effective_dating_constraint(blocks: ReadRequestBuildingBlocks, res: Re
     if r.effective_dating is None:
         return
 
+    op1, op2 = __get_range_semantics_ops(blocks.effective_dating_range_semantics)
+
     if blocks.effective_dating_table is not None:
         blocks.constraints.append(
-            blocks.effective_dating_table.c.dt >= a.c.effective_start_dt
+            blocks.effective_dating_table.c.dt.op(op1)(a.c.effective_start_dt)
         )
 
         blocks.constraints.append(
-            blocks.effective_dating_table.c.dt < a.c.effective_end_dt
+            blocks.effective_dating_table.c.dt.op(op2)(a.c.effective_end_dt)
         )
 
         blocks.constraints.append(
-            blocks.effective_dating_table.c.dt >= datetime_to_date_string(blocks.effective_start_dt)
+            blocks.effective_dating_table.c.dt.op(op1)(datetime_to_date_string(blocks.effective_start_dt))
         )
 
         blocks.constraints.append(
-            blocks.effective_dating_table.c.dt < datetime_to_date_string(blocks.effective_end_dt)
+            blocks.effective_dating_table.c.dt.op(op2)(datetime_to_date_string(blocks.effective_end_dt))
         )
-
     else:
         blocks.constraints.append(
-            a.c.effective_start_dt
-            <= timestamp_to_date_string(
+            a.c.effective_start_dt.op(op1)(timestamp_to_date_string(
                 res.effective_dating.end_dt.seconds
-            )
+            ))
         )
         blocks.constraints.append(
-            a.c.effective_end_dt
-            >= timestamp_to_date_string(
+            a.c.effective_end_dt.op(op2)(timestamp_to_date_string(
                 res.effective_dating.start_dt.seconds
-            )
+            ))
         )
 
 
 def __add_valid_dating_constraint(blocks: ReadRequestBuildingBlocks, res: ResolveResourceIntentResponse):
     a = blocks.asset_table
 
+    op1, op2 = __get_range_semantics_ops(blocks.valid_dating_range_semantics)
+
     if blocks.valid_dating_table is not None:
         blocks.constraints.append(
-            blocks.valid_dating_table.c.dt >= a.c.valid_start_dt
+            blocks.valid_dating_table.c.dt.op(op1)(a.c.valid_start_dt)
         )
 
         blocks.constraints.append(
-            blocks.valid_dating_table.c.dt < a.c.valid_end_dt
+            blocks.valid_dating_table.c.dt.op(op2)(a.c.valid_end_dt)
         )
 
         blocks.constraints.append(
-            blocks.valid_dating_table.c.dt >= datetime_to_date_string(blocks.valid_start_dt)
+            blocks.valid_dating_table.c.dt.op(op1)(datetime_to_date_string(blocks.valid_start_dt))
         )
 
         blocks.constraints.append(
-            blocks.valid_dating_table.c.dt < datetime_to_date_string(blocks.valid_end_dt)
+            blocks.valid_dating_table.c.dt.op(op2)(datetime_to_date_string(blocks.valid_end_dt))
         )
 
 
-def __populate_building_blocks_dating(request_field: str, start_field: str, end_field: str, dating_tbl_field: str,
+def __populate_building_blocks_dating(request_field: str, start_field: str, end_field: str, dating_tbl_field: str, semantics_field: str,
                                       helper: DataStoreHelper, blocks: ReadRequestBuildingBlocks, r: ReadRequest):
     if not hasattr(r, request_field):
         raise RuntimeError('Invalid field name.')
@@ -202,14 +216,16 @@ def __populate_building_blocks_dating(request_field: str, start_field: str, end_
             setattr(blocks, start_field, start)
             setattr(blocks, end_field, end)
 
-            blocks.effective_start_dt = start
-            blocks.effective_end_dt = end
-
         elif len(dating) == 3:
 
             setattr(blocks, dating_tbl_field, helper.get_series_table(dating))
             setattr(blocks, start_field, dating[0])
             setattr(blocks, end_field, dating[1])
+        elif len(dating) == 4:
+            setattr(blocks, dating_tbl_field, helper.get_series_table(dating))
+            setattr(blocks, start_field, dating[0])
+            setattr(blocks, end_field, dating[1])
+            setattr(blocks, semantics_field, dating[3])
 
         else:
             raise RuntimeError('Invalid dating constraint.')
@@ -281,6 +297,8 @@ def __build_read_query(
             datetime_to_date_string(r.knowledge_dt) >= a.c.knowledge_start_dt,
             datetime_to_date_string(r.knowledge_dt) < a.c.knowledge_end_dt,
         ])],
+        valid_dating_range_semantics='[)',
+        effective_dating_range_semantics='[)'
     )
 
     __populate_building_blocks_dating(
@@ -288,6 +306,7 @@ def __build_read_query(
         'effective_start_dt',
         'effective_end_dt',
         'effective_dating_table',
+        'effective_dating_range_semantics',
         store_helper,
         blocks,
         r
@@ -298,6 +317,7 @@ def __build_read_query(
         'valid_start_dt',
         'valid_end_dt',
         'valid_dating_table',
+        'valid_dating_range_semantics',
         store_helper,
         blocks,
         r
